@@ -2,13 +2,14 @@
 library(testthat)
 library(synthACS)
 
+load("C:/Github_projects/ACSpulls/synthACS/tests/testthat/acsdat.Rdata")
+
 #----------------------------------------------------------
 context("adding constraints")
 #----------------------------------------------------------
 
 test_that("constraint errors work", {
   # geography
-  load("C:/Github_projects/ACSpulls/synthACS/tests/testthat/acsdat.Rdata")
   data(diamonds, package= "ggplot2")
   let <- letters
   
@@ -23,9 +24,6 @@ test_that("constraint errors work", {
 
 
 test_that("adding constraints work", {
-  # geography
-  load("C:/Github_projects/ACSpulls/synthACS/tests/testthat/acsdat.Rdata")
-  
   # test -- add some constraints singly
   expect_true(is.list(add_constraint(attr_name= "age", attr_totals= a, micro_data= test_micro)))
   expect_equal(length(add_constraint(attr_name= "age", attr_totals= a, micro_data= test_micro)), 1)
@@ -68,8 +66,6 @@ context("calculating TAE - helpers")
 #----------------------------------------------------------
 
 test_that("sample_totals calculates accurately", {
-  load("C:/Github_projects/ACSpulls/synthACS/tests/testthat/acsdat.Rdata")
-  
   # build out constraints
   c_list <- add_constraint(attr_name= "age", attr_totals= a, micro_data= test_micro)
   c_list <- add_constraint(attr_name= "edu_attain", attr_totals= e, micro_data= test_micro,
@@ -91,8 +87,6 @@ test_that("sample_totals calculates accurately", {
 })
 
 test_that("tae_mapply calculates accurately", {
-  load("C:/Github_projects/ACSpulls/synthACS/tests/testthat/acsdat.Rdata")
-  
   # build out constraints
   c_list <- add_constraint(attr_name= "age", attr_totals= a, micro_data= test_micro)
   c_list <- add_constraint(attr_name= "edu_attain", attr_totals= e, micro_data= test_micro,
@@ -118,9 +112,6 @@ context("calculating TAE")
 #----------------------------------------------------------
 
 test_that("tae errors work appropriately", {
-  # geography
-  load("C:/Github_projects/ACSpulls/synthACS/tests/testthat/acsdat.Rdata")
-
   # build out constraints
   c_list <- add_constraint(attr_name= "age", attr_totals= a, micro_data= test_micro)
   c_list <- add_constraint(attr_name= "edu_attain", attr_totals= e, micro_data= test_micro,
@@ -144,9 +135,6 @@ test_that("tae errors work appropriately", {
 })
 
 test_that("TAE results are exactly correct", {
-  # geography
-  load("C:/Github_projects/ACSpulls/synthACS/tests/testthat/acsdat.Rdata")
-  
   # build out constraints
   c_list <- add_constraint(attr_name= "age", attr_totals= a, micro_data= test_micro)
   c_list <- add_constraint(attr_name= "edu_attain", attr_totals= e, micro_data= test_micro,
@@ -177,9 +165,6 @@ test_that("TAE results are exactly correct", {
 
 
 test_that("TAE ouptuts appropriately - no iterations", {
-  # geography
-  load("C:/Github_projects/ACSpulls/synthACS/tests/testthat/acsdat.Rdata")
-  
   # build out constraints
   c_list <- add_constraint(attr_name= "age", attr_totals= a, micro_data= test_micro)
   c_list <- add_constraint(attr_name= "edu_attain", attr_totals= e, micro_data= test_micro,
@@ -203,9 +188,6 @@ test_that("TAE ouptuts appropriately - no iterations", {
 })
 
 test_that("TAE results are exactly correct - iterations 2-3", {
-  # geography
-  load("C:/Github_projects/ACSpulls/synthACS/tests/testthat/acsdat.Rdata")
-  
   # build out constraints
   c_list <- add_constraint(attr_name= "age", attr_totals= a, micro_data= test_micro)
   c_list <- add_constraint(attr_name= "edu_attain", attr_totals= e, micro_data= test_micro,
@@ -282,9 +264,6 @@ test_that("TAE results are exactly correct - iterations 2-3", {
 })
 
 test_that("TAE outputs appropriately - iterations 2-3", {
-  # geography
-  load("C:/Github_projects/ACSpulls/synthACS/tests/testthat/acsdat.Rdata")
-  
   # build out constraints
   c_list <- add_constraint(attr_name= "age", attr_totals= a, micro_data= test_micro)
   c_list <- add_constraint(attr_name= "edu_attain", attr_totals= e, micro_data= test_micro,
@@ -344,7 +323,6 @@ context("simulated annealing")
 test_that("errors work", {
   # build up some test data:
   data(diamonds, package= "ggplot2")
-  load("C:/Github_projects/ACSpulls/synthACS/tests/testthat/acsdat.Rdata")
   
   # build out constraints
   c_list <- add_constraint(attr_name= "age", attr_totals= a, micro_data= test_micro)
@@ -395,9 +373,6 @@ test_that("errors work", {
 
 
 test_that("annealing works correctly", {
-  # geography
-  load("C:/Github_projects/ACSpulls/synthACS/tests/testthat/acsdat.Rdata")
-  
   # build out constraints
   c_list <- add_constraint(attr_name= "age", attr_totals= a, micro_data= test_micro)
   c_list <- add_constraint(attr_name= "edu_attain", attr_totals= e, micro_data= test_micro,
@@ -411,7 +386,8 @@ test_that("annealing works correctly", {
   c_list <- c_list <- add_constraint(attr_name= "nativity", attr_totals= n, micro_data= test_micro,
                                      constraint_list= c_list)
   # sample data and run
-  anneal1 <- optimize_microdata(test_micro, "p", c_list, max_iter= 10, resample_size= 500, p_accept= 0.01)
+  anneal1 <- optimize_microdata(test_micro, "p", c_list, max_iter= 10, resample_size= 500, p_accept= 0.01,
+                                verbose= FALSE)
   
   # test structure of output
   expect_true(is.list(anneal1))
@@ -419,9 +395,17 @@ test_that("annealing works correctly", {
   expect_true(is.data.table(anneal1$best_fit))
   expect_true(is.numeric(anneal1$tae))
   expect_true(is.numeric(anneal1$p_accept))
+  expect_equal(anneal1$p_accept, 0.01)
   expect_true(is.numeric(anneal1$iter))
   expect_true(is.numeric(anneal1$max_iter))
   expect_lte(anneal1$iter, anneal1$max_iter)
+  expect_equal(anneal1$max_iter, 10L)
+  
+  expect_true(is.matrix(anneal1$tae_path))
+  expect_equal(nrow(anneal1$tae_path), anneal1$iter)
+  expect_equal(ncol(anneal1$tae_path), 2L)
+  expect_equal(anneal1$seed %% 1, 0)
+  expect_true(all(anneal1$tae_path[,1] >= anneal1$tae_path[,2]))
   
   expect_true(all(unlist(lapply(anneal1$best_fit, is.factor))))
   expect_true(all(names(anneal1$best_fit) %in% names(test_micro)))
